from textwrap import wrap
from PIL import (
    ImageDraw,
    ImageFont,
)


def draw_text_on_image(
        image,
        text,
        font,
        font_size,
        font_color,
        padding,
):
    """
    Draw text on center image.
    !!! Warning. Image argument is not immutable.
    """
    width, height = image.size

    font = ImageFont.truetype(font, size=font_size, encoding='UTF-8')
    line_length = int((width * (1 - padding)) / (font.getsize(text)[0] / len(text)))
    lines = wrap(text, line_length)

    draw = ImageDraw.Draw(image)

    text_width, text_height = font.getsize_multiline('\n'.join(lines))
    y_text = height / 2 - (text_height / 2)

    for line in lines:
        font_width, font_height = font.getsize(line)
        draw.text(((width - font_width) / 2, y_text), line, font=font, fill=font_color)
        y_text += font_height

    return image



from io import BytesIO
from PIL import (
    Image,
    ImageEnhance,
)

from .size import resize_image_for_snippet
from .text import draw_text_on_image


def create_snippet_image(
        font,
        size=None,
        text='',
        background=None,
        background_color=None,
        overlay=None,
        brightness=None,
        font_color=None,
        font_size=64,
        padding=0.1,
        center=None,
):
    """
    Creates images with a combined cropped background and overlay images and adds text in the center.

    :param font: Path to font file. Is required. For load font used PIL.ImageFont.
    :type font: str
    :param size: Size of snippet image. tuple(width, height).
    :type size: tuple(int, int)
    :param text: Text of snippet image. By default is an empty string.
    :type text: str
    :param background: Path to background image file.
    :type background: str
    :param background_color: Background color of snippet image. Used when background is None.
    :type background_color: tuple(int, int, int)
    :param overlay: Path to overlay image. if size is None, overlay size is used.
                    As an overlay, an image with a transparent background is used.
    :type overlay: str
    :param brightness: Brightness of background of snippet image. Value from 0 to 1.
    :type brightness: float
    :param font_color: Font color in RGBA. By default is (255, 255, 255, 255).
    :type font_color: tuple(int, int, int, int)
    :param font_size: Size of snippet image text. By default is 64.
    :type font_size: int
    :param padding: Text indents to the left and right of the snippet image.
                    Value from 0 to 1.
                    0 - 0% width;
                    1 - 100% width.
    :type padding: float
    :param center : Background image center for crop and resize image. tuple(x, y).
                    Defaults is center of background image.
    :type center: tuple(int, int)

    :return image: BytesIO blob of snippet image,
    :rtype: BytesIO
    """
    background_color = background_color or (0, 0, 0)
    font_color = font_color or (255, 255, 255, 255)

    if not size and not overlay:
        raise TypeError('Overlay and size can not be empty at the same time.')

    overlay_image = overlay and Image.open(overlay)
    size = size or overlay_image.size

    image = get_background_image(
        size,
        background=background,
        background_color=background_color,
        brightness=brightness,
    )

    image = resize_image_for_snippet(image, size, center)

    image = draw_text_on_image(
        image,
        text,
        font,
        font_size,
        font_color,
        padding=padding,
    )

    if overlay_image:
        image.paste(overlay_image, (0, 0), mask=overlay_image)

    image_blob = get_image_blob(image)

    return image_blob


def get_background_image(
        size,
        brightness=None,
        background=None,
        background_color=None,
):
    image_mode = 'RGB'

    if background:
        image = Image.open(background)
    else:
        image = Image.new(image_mode, size, background_color)

    image = image.convert(image_mode) if not image.mode == image_mode else image

    if brightness:
        image = ImageEnhance.Brightness(image).enhance(brightness)

    return image


def get_image_blob(image):
    blob = BytesIO()
    image.save(blob, 'JPEG')

    return blob



def resize_image_for_snippet(
        image,
        size,
        center=None,
):
    """
    Crop and resize the image depending on the center and size.
    !!! Warning. Image argument is not immutable.
    """
    image = crop_image(image, size, center)
    image = resize_image(image, size)

    return image


def resize_image(image, size):
    return image.resize(size)


def crop_image(image, size, center=None):
    if center and (not isinstance(center, tuple) or not len(center) == 2):
        raise ValueError('Center must be tuple with 2 elements - tuple(x, y).')

    image_width, image_height = image.size
    width, height = size
    center = center or (int(image_width / 2), int(image_height / 2))

    if image_width > width or image_height > height:
        half_width = int(width / 2)
        half_height = int(height / 2)
        center_x, center_y = center

        left = center_x - half_width
        right = center_x + half_width
        top = center_y - half_height
        bottom = center_y + half_height

        left, right = correct_size(
            left,
            right,
            0,
            image_width,
        )

        top, bottom = correct_size(
            top,
            bottom,
            0,
            image_height,
        )

        image = image.crop((left, top, right, bottom))

    return image


def correct_size(
        min_value,
        max_value,
        min_purpose_value,
        max_purpose_value,
):
    if min_value < min_purpose_value:
        correction = min_purpose_value - min_value
        min_value = min_purpose_value
        max_value = max_value + correction

        if max_value > max_purpose_value:
            max_value = max_purpose_value
    elif max_value > max_purpose_value:
        correction = max_purpose_value - max_value
        max_value = max_purpose_value
        min_value = min_value + correction

        if min_value < min_purpose_value:
            min_value = min_purpose_value

    return min_value, max_value



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['snippet_image']

package_data = \
{'': ['*']}

install_requires = \
['pillow>=5.2,<6.0']

setup_kwargs = {
    'name': 'snippet-image',
    'version': '0.1.2',
    'description': 'Package for simple creation images of snippets for social networks using pillow. Pictures for snippets can be inserted for sharing pages in social networks, for example in tag meta:',
    'long_description': '============================\nsnippet-image\n============================\n\nPackage for simple creation images of snippets for social networks using pillow_.\n\n.. _pillow: https://pillow.readthedocs.io/en/stable/\n\nPictures for snippets can be inserted for sharing pages in social networks, for example in tag meta:\n\n.. code-block:: html\n\n    <meta property="og:image" content="Link to your snippet image" />\n    <meta name="twitter:image" content="Link to your snippet image" />\n\nInstallation\n---------------------------\n\n`pip3 install snippet-image`\n\nUser guide\n---------------------------\n\nTo start creating images for snippets, it is enough to import the function ```from snippet_image import create_snippet_image```.\n\n.. code-block:: python\n\n    from snippet_image import create_snippet_image\n\n    image_blob = create_snippet_image(\n            font=\'/home/iamterminator/.fonts/OpenSans-Bold.ttf\', # Path to your font file\n            font_size=62, # Font size\n            background=\'/home/iamterminator/.wallpapers/jakethedog.jpg\', # Path to your background image\n            size=(1200, 630), # Size of snippet image. (width, height)\n            text=\'Jake the Dog\', # Text for draw on snippet image\n            brightness=0.3, # Brightness of background\n        )\n\n    with open(\'jake-the-dog-snippet-image.jpg\', \'wb\') as file:\n        file.write(image_blob.getvalue())\n\nRead more on home_.\n\n.. _home: https://github.com/acrius/snippet-image.\n',
    'author': 'acrius',
    'author_email': 'acrius@mail.ru',
    'url': 'https://github.com/acrius/snippet-image',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
